package cn.tonyandmoney.tuon.core.error;

import com.fasterxml.jackson.annotation.JsonInclude;

/**
 * 返回前端的基础数据，如果code为 0 ，则表示返回正常，不为0 则表示操作异常
 *
 * @param <T> 数据
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
public class BaseResp<T> {

    private int code = 0;
    private String message = "success";

    private T data;

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getMessage() {
        return message;
    }

    public BaseResp<T> setMessage(String message) {
        this.message = message;
        return this;
    }

    public T getData() {
        return data;
    }

    public BaseResp<T> setData(T data) {
        this.data = data;
        return this;
    }

    public static <V> BaseResp<V> ok(V data) {
        return new BaseResp<V>()
                .setData(data);
    }


    public static <V> BaseResp<V> error(Throwable error) {
        BaseResp<V> resp = new BaseResp<>();
        resp.setCode(-1);
        resp.setMessage(error.getMessage() == null ? "服务器运行异常！" : error.getMessage());
        return resp;
    }
}
