package cn.tonyandmoney.tuon.core.session;

import cn.tonyandmoney.tuon.core.error.SessionNoUserException;
import cn.tonyandmoney.tuon.core.user.IUser;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

/**
 * 将身份信息注入的线程的本地数据中
 *
 * @Classname SessionUtils
 * @Description TODO
 * @Date 2019/8/25 19:51
 * @Created by Administrator
 */
public class SessionUtils {

    public static final String USER_ID="userId";
    public static final String USER="user";

    private static ThreadLocal<IUser> mUserData = new ThreadLocal<>();


    /**
     * 设置用户信息
     *
     * @param user 用户信息
     */
    public static void setUser(IUser user) {
        mUserData.set(user);
    }


    /**
     * 清楚信息
     */
    public static void remove() {
        mUserData.remove();
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    public static IUser getUser() {
        return mUserData.get();
    }

    /**
     * 是否登录
     *
     * @return 由用户信息，则表示一登录
     */
    public static boolean isLogin() {
        return mUserData.get() != null;
    }



    /**
     * 异步获取用户信息，如果当前没有用户信息，则抛出异常
     * 一下三个方法是为WebFlux 服务下实现的
     *
     * @param exchange 这个
     * @return
     */
    public static <T extends IUser> Mono<T> getUser(ServerWebExchange exchange) {
        return exchange.getSession()
                .flatMap(webSession -> {
                    IUser info = webSession.getAttribute(USER);
                    if (info != null) {
                        return Mono.just((T) info);
                    }
                    return Mono.error(new SessionNoUserException());
                });
    }

    public static Mono<String> getUserId(ServerWebExchange exchange) {
        return exchange.getSession()
                .flatMap(webSession -> {
                    String userId = webSession.getAttribute(USER_ID);
                    if (StringUtils.isBlank(userId)) {
                        return Mono.error(new SessionNoUserException());
                    }
                    return Mono.just(userId);
                });
    }


    public static Mono<IUser> setUser(ServerWebExchange exchange, IUser user) {
        return exchange.getSession()
                .flatMap(webSession -> {
                    webSession.getAttributes().put(USER, user);
                    webSession.getAttributes().put(USER_ID, user.getUserId());
                    return Mono.just(user);
                });
    }



}
