package cn.tonyandmoney.tuon.core.session.filter;

import cn.tonyandmoney.tuon.core.session.SessionUtils;
import cn.tonyandmoney.tuon.core.user.IUser;
import org.springframework.core.Ordered;
import org.springframework.session.web.http.SessionRepositoryFilter;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;

/**
 * session 注入和移除
 */
public class SessionFillFilter extends OncePerRequestFilter implements Ordered {

    /**
     * The default filter order.
     * 一定要比 SessionRepositoryFilter 的顺序靠后，不然怎么可能获取到session呢
     */
    public static final int DEFAULT_ORDER = Integer.MIN_VALUE + 51;


    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {

        try {
            HttpSession session = request.getSession(false);
            if (session != null) {
                Object user = session.getAttribute(SessionUtils.USER);
                if (user != null) {
                    SessionUtils.setUser((IUser) user);
                }
            }
            filterChain.doFilter(request, response);
        } finally {
            SessionUtils.remove();
        }

    }

    @Override
    public int getOrder() {
        return DEFAULT_ORDER;
    }
}
