package cn.tonyandmoney.tuon.core.batis

import cn.gygxzc.envir.core.orm.injector.LogicMoreSqlInjector
import cn.tonyandmoney.tuon.core.batis.handler.CustomMetaObjectHandler
import cn.tonyandmoney.tuon.core.properties.BatisMetaProperties
import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler
import com.baomidou.mybatisplus.core.injector.ISqlInjector
import com.baomidou.mybatisplus.extension.plugins.PaginationInterceptor
import com.baomidou.mybatisplus.extension.plugins.PerformanceInterceptor
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.Profile
import org.springframework.transaction.annotation.EnableTransactionManagement

/**
 *
 * @author NorthLan at 2018/1/30
 * mybatis plus 配置
 */
@Configuration
@EnableTransactionManagement
@EnableConfigurationProperties(value = [BatisMetaProperties::class])
class MybatisPlusConfig {


    @Autowired
    private lateinit var mProperties: BatisMetaProperties

    /**
     * 并不会自动注入
     */
    @Bean
    fun physicalSqlInjector(): ISqlInjector {
        return LogicMoreSqlInjector()
    }


    /**
     * 性能分析插件
     * // 开发环境开启
     */
    @Bean
    @Profile("dev")
    fun performanceInterceptor(): PerformanceInterceptor {
        return PerformanceInterceptor().also { it.isFormat = true }
    }

    /**
     * 分页插件
     */
    @Bean
    fun paginationInterceptor(): PaginationInterceptor {
        return PaginationInterceptor()
    }

    @Bean
    fun metaObjectHandler(): MetaObjectHandler {
        return CustomMetaObjectHandler(mProperties)
    }

}