package cn.tonyandmoney.tuon.core.batis.handler

import cn.tonyandmoney.tuon.core.properties.BatisMetaProperties
import cn.tonyandmoney.tuon.core.session.SessionUtils
import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler
import org.apache.ibatis.reflection.MetaObject
import org.slf4j.LoggerFactory
import java.util.*

/**
 * 公共字段自动填充
 * 多线程 + WebSocket 下不适用
 * @author NorthLan
 * @date 2018/1/31
 * @url https://noahlan.com
 */
class CustomMetaObjectHandler(private val properties: BatisMetaProperties) : MetaObjectHandler {
    companion object {
        val logger = LoggerFactory.getLogger(CustomMetaObjectHandler::class.java)!!
    }

    /**
     * 公告字段插入的时候填充数据
     */
    override fun insertFill(metaObject: MetaObject?) {
        try {
            if (getFieldValByName(properties.createBy, metaObject) == null) {
                val curUser = SessionUtils.getUser()?.account
                setFieldValByName(properties.createBy, curUser, metaObject)
            }
            if (getFieldValByName(properties.updateBy, metaObject) == null) {
                setFieldValByName(properties.updateBy, Date(), metaObject)
            }
        } catch (e: Throwable) {
            logger.warn("insertFill error, cause: ", e)
        }
    }

    /**
     * 更新时间注入信息
     */
    override fun updateFill(metaObject: MetaObject?) {
        try {
            if (getFieldValByName(properties.updateBy, metaObject) == null) {
                val curUser = SessionUtils.getUser()?.account
                setFieldValByName(properties.updateBy, curUser, metaObject)
            }
            if (getFieldValByName(properties.updateTime, metaObject) == null) {
                setFieldValByName(properties.updateTime, Date(), metaObject)
            }
        } catch (e: Throwable) {
            logger.warn("updateFill error, cause: ", e)
        }
    }
}